@@ -1,3 +1,4 @@
+from itertools import zip_longest
 import math
 from dataclasses import dataclass
 
@@ -5,15 +6,24 @@ import torch
 from torch import nn
 from torch.autograd import Function
 
+from lightseq.training.ops.pytorch import TransformerBuilder
 from lightseq.training.ops.pytorch.builder import TransformerBuilder
 from lightseq.training.ops.pytorch.util import (
     copy_para,
     state_dict,
     MODEL_ARCH,
     check_config,
+    calc_offset,
 )
+from lightseq.training.ops.pytorch.quantization import (
+    weight_quant_config,
+    act_quant_config,
+    relu_quant_config,
+)
+from lightseq.training.ops.pytorch.layer_base import TransformerDecoderLayerBase
+
+transformer_cuda_module = TransformerBuilder().load()
 
-transformer_cuda_module = None
 _all_layer_grads = dict()
 _shared_encdec_attn_kv_params = dict()
 
@@ -43,6 +53,7 @@ class LSTransformerDecoderFunc(Function):
             encoder_padding_mask,
             config.training,
             config.pre_layer_norm,
+            config.quant_mode,
             cache,
         )
 
@@ -97,7 +108,7 @@ class LSTransformerDecoderFunc(Function):
         return (grad_input, grad_enc_out, None, grad, None, None)
 
 
-class LSTransformerDecoderLayer(nn.Module):
+class LSTransformerDecoderLayer(TransformerDecoderLayerBase):
     """Initialize the Lightseq Transformer Decoder Layer.
 
     Static variable:
@@ -122,14 +133,11 @@ class LSTransformerDecoderLayer(nn.Module):
 
         print("Lightseq Transformer config is ", self.config.__dict__)
 
+        self.quant_mode = False
+
         if self.config.local_rank >= 0:
             torch.cuda.set_device(self.config.local_rank)
 
-            # Load cuda modules if needed
-        global transformer_cuda_module
-        if transformer_cuda_module is None:
-            transformer_cuda_module = TransformerBuilder().load()
-
         # create the layer in cuda kernels.
         cuda_module = transformer_cuda_module
         create_layer_func = (
@@ -154,7 +162,55 @@ class LSTransformerDecoderLayer(nn.Module):
 
         hs = self.config.hidden_size
         ims = self.config.intermediate_size
-        nums = [
+        self.hs = hs
+        self.ims = ims
+
+        self.para_offset = LSTransformerDecoderLayer.gen_offset(
+            hs, ims, self.config.nlayer
+        )
+        if self.config.layer_id != 0:
+            self.para_offset = self.para_offset[:-2]
+        self.para = nn.Parameter(torch.Tensor(self.para_offset[-1]))
+
+        if initial_weights is None or initial_biases is None:
+            # enc-dec kv weights and bias
+            self.init_transformer_weights()
+            return
+
+        # For testing only.
+        attn_qkvw = [ele.detach().clone() for ele in initial_weights[:3]]
+        attn_qkvw = torch.cat(attn_qkvw, dim=0)
+        weights = [attn_qkvw] + [
+            copy_para(ele) if ele is not None else None for ele in initial_weights[3:]
+        ]
+
+        attn_qkvb = [ele.detach().clone() for ele in initial_biases[:3]]
+        attn_qkvb = torch.cat(attn_qkvb, dim=0)
+        biases = [attn_qkvb] + [
+            copy_para(ele) if ele is not None else None for ele in initial_biases[3:]
+        ]
+
+        idx = 0
+        for w, b in zip_longest(weights, biases):
+            if w is not None:
+                cur_para = self._get_weights(idx)
+                assert cur_para.numel() == w.numel()
+                cur_para.copy_(w.view(-1))
+                idx += 1
+
+            if b is not None:
+                cur_para = self._get_weights(idx)
+                assert cur_para.numel() == b.numel()
+                cur_para.copy_(b.view(-1))
+                idx += 1
+
+    @staticmethod
+    def gen_offset(hidden_size, intermediate_size, nlayer):
+        """Returns the offset of each module's parameters among all
+        parameters of a layer
+        """
+        hs, ims = hidden_size, intermediate_size
+        sizes = [
             hs * hs * 3,  # attn_qkvw
             hs * 3,  # attn_qkvb
             hs * hs,  # attn_ow
@@ -173,72 +229,86 @@ class LSTransformerDecoderLayer(nn.Module):
             hs,  # output_b
             hs,  # ffn_nw
             hs,  # ffn_nb
+            24,
+            hs * hs * 2 * nlayer,  # encdec_attn_kvw
+            hs * 2 * nlayer,  # encdec_attn_kvb
         ]
+        offsets = calc_offset(sizes)
+        return offsets
+
+    def params_dict(self):
+        """
+        Returns:
+            weight: dict
+            bias: dict
+        """
+
+        def copy_and_view(m, shape=None):
+            if shape is None:
+                shape = (-1,)
+            return m.data.clone().view(*shape)
+
+        def _copy(m):
+            return copy_and_view(m, (self.hs, self.hs))
+
+        self_attn_qkvw = self._get_weights(0)
+        self_attn_qw, self_attn_kw, self_attn_vw = self_attn_qkvw.split(
+            self.hs * self.hs, 0
+        )
+        self_attn_qkvb = self._get_weights(1)
+        self_attn_qb, self_attn_kb, self_attn_vb = self_attn_qkvb.split(self.hs, 0)
+
+        all_enc_attn_kw, all_enc_attn_vw = None, None
+        all_enc_attn_kb, all_enc_attn_vb = None, None
         if self.config.layer_id == 0:
-            nums += [
-                hs * hs * 2 * self.config.nlayer,  # encdec_attn_kvw
-                hs * 2 * self.config.nlayer,  # encdec_attn_kvb
-            ]
-        offset = 0
-        self.para_offset = [offset]
-        for n in nums:
-            offset += n
-            self.para_offset.append(offset)
-        self.para = nn.Parameter(torch.Tensor(offset))
+            all_enc_attn_kvw = self._get_weights(19)
+            all_enc_attn_kvw = all_enc_attn_kvw.split(self.hs * self.hs, 0)
+            all_enc_attn_kw = list(map(_copy, all_enc_attn_kvw[::2]))
+            all_enc_attn_vw = list(map(_copy, all_enc_attn_kvw[1::2]))
 
-        if initial_weights is None and initial_biases is None:
-            # enc-dec kv weights and bias
-            self.init_transformer_weights()
-            return
+            all_enc_attn_kvb = self._get_weights(20)
+            all_enc_attn_kvb = all_enc_attn_kvb.split(self.hs, 0)
+            all_enc_attn_kb = list(map(copy_and_view, all_enc_attn_kvb[::2]))
+            all_enc_attn_vb = list(map(copy_and_view, all_enc_attn_kvb[1::2]))
 
-        # For testing only.
-        attn_qkvw = [ele.detach().clone() for ele in initial_weights[:3]]
-        attn_qkvw = torch.cat(attn_qkvw, dim=0)
-        weights = [attn_qkvw] + [copy_para(ele) for ele in initial_weights[3:]]
-
-        attn_qkvb = [ele.detach().clone() for ele in initial_biases[:3]]
-        attn_qkvb = torch.cat(attn_qkvb, dim=0)
-        biases = [attn_qkvb] + [copy_para(ele) for ele in initial_biases[3:]]
-
-        idx = 0
-        for w, b in zip(weights, biases):
-            cur_para = self._get_weights(idx)
-            assert cur_para.numel() == w.numel()
-            cur_para.copy_(w.view(-1))
-            idx += 1
-
-            cur_para = self._get_weights(idx)
-            assert cur_para.numel() == b.numel()
-            cur_para.copy_(b.view(-1))
-            idx += 1
-
-    @staticmethod
-    def get_config(**kwargs):
-        @dataclass
-        class Config:
-            max_batch_tokens: int  # max batch token numbers
-            max_seq_len: int  # max sequence length
-            hidden_size: int  # size of transformer hidden layers
-            intermediate_size: int  # size of ffn inner size
-            nhead: int  # number of heads in attention
-            attn_prob_dropout_ratio: float  # attention score dropout ratio
-            activation_dropout_ratio: float  # ffn activation dropout ratio
-            hidden_dropout_ratio: float  # dropout ration before residual
-            pre_layer_norm: bool  # pre layer norm or post
-            fp16: bool  # fp16 presion
-            local_rank: int  # rank in local node
-            nlayer: int  # number of layers
-            activation_fn: str = "relu"  # relu or gelu
-
-        if "model" in kwargs:
-            if kwargs["model"] not in MODEL_ARCH:
-                raise ValueError("{} architecture is not supported.")
-            MODEL_ARCH[kwargs["model"]](kwargs)
-            del kwargs["model"]
-
-        config = Config(**kwargs)
-        check_config(config)
-        return config
+        weight = {
+            "self_attn.q_proj": copy_and_view(self_attn_qw, (self.hs, self.hs)),
+            "self_attn.k_proj": copy_and_view(self_attn_kw, (self.hs, self.hs)),
+            "self_attn.v_proj": copy_and_view(self_attn_vw, (self.hs, self.hs)),
+            "self_attn.out_proj": copy_and_view(
+                self._get_weights(2), (self.hs, self.hs)
+            ),
+            "self_attn_layer_norm": copy_and_view(self._get_weights(4), (self.hs,)),
+            "encoder_attn.q_proj": copy_and_view(
+                self._get_weights(6), (self.hs, self.hs)
+            ),
+            "encoder_attn.out_proj": copy_and_view(
+                self._get_weights(8), (self.hs, self.hs)
+            ),
+            "encoder_attn_layer_norm": copy_and_view(self._get_weights(10), (self.hs,)),
+            "fc1": copy_and_view(self._get_weights(12), (self.ims, self.hs)),
+            "fc2": copy_and_view(self._get_weights(14), (self.hs, self.ims)),
+            "final_layer_norm": copy_and_view(self._get_weights(16), (self.hs,)),
+            "clip_max": copy_and_view(self._get_weights(18), (24,)),
+            "encoder_attn.k_proj": all_enc_attn_kw,
+            "encoder_attn.v_proj": all_enc_attn_vw,
+        }
+        bias = {
+            "self_attn.q_proj": copy_and_view(self_attn_qb),
+            "self_attn.k_proj": copy_and_view(self_attn_kb),
+            "self_attn.v_proj": copy_and_view(self_attn_vb),
+            "self_attn.out_proj": copy_and_view(self._get_weights(3)),
+            "self_attn_layer_norm": copy_and_view(self._get_weights(5)),
+            "encoder_attn.q_proj": copy_and_view(self._get_weights(7), (self.hs,)),
+            "encoder_attn.out_proj": copy_and_view(self._get_weights(9), (self.hs,)),
+            "encoder_attn_layer_norm": copy_and_view(self._get_weights(11), (self.hs,)),
+            "fc1": copy_and_view(self._get_weights(13)),
+            "fc2": copy_and_view(self._get_weights(15)),
+            "final_layer_norm": copy_and_view(self._get_weights(17)),
+            "encoder_attn.k_proj": all_enc_attn_kb,
+            "encoder_attn.v_proj": all_enc_attn_vb,
+        }
+        return weight, bias
 
     def _get_weights(self, i):
         return self.para.data.narrow(
@@ -246,6 +316,7 @@ class LSTransformerDecoderLayer(nn.Module):
         )
 
     def calc_bound(self, w):
+        """Used to initialize parameters"""
         fan_in, _ = nn.init._calculate_fan_in_and_fan_out(w)
         bound = 1.0 / math.sqrt(fan_in)
         return bound
@@ -291,13 +362,19 @@ class LSTransformerDecoderLayer(nn.Module):
         nn.init.ones_(self._get_weights(16))
         nn.init.zeros_(self._get_weights(17))
 
+        act_cmax = act_quant_config.amax.tolist()
+        wei_cmax = weight_quant_config.amax.tolist()
+        init_clip_max = torch.tensor([act_cmax, wei_cmax, act_cmax] * 8)
+        self._get_weights(18).copy_(init_clip_max)
+
         if self.config.layer_id == 0:
-            encdec_attn_kvw = self._get_weights(18).view(-1, hs)
+            encdec_attn_kvw = self._get_weights(19).view(-1, hs)
             nn.init.xavier_uniform_(encdec_attn_kvw, 1.0 / math.sqrt(2.0))
             bound = self.calc_bound(encdec_attn_kvw)
-            nn.init.uniform_(self._get_weights(19), -bound, bound)
+            nn.init.uniform_(self._get_weights(20), -bound, bound)
 
     def __assign_layer_weight_grad(self):
+        """fp16 or fp32"""
         param = (
             self.para_16
             if self.config.fp16 and self.para.dtype != torch.half
@@ -306,7 +383,7 @@ class LSTransformerDecoderLayer(nn.Module):
 
         if self.config.layer_id in _all_layer_grads:
             return
-        grad = torch.empty_like(param)
+        grad = torch.zeros_like(param)
         cuda_module = transformer_cuda_module
         if self.config.fp16:
             func = cuda_module.assign_layer_weight_grad_fp16
@@ -315,29 +392,29 @@ class LSTransformerDecoderLayer(nn.Module):
         func(param, grad, "TransformerDecoderLayer", self.config.layer_id)
         _all_layer_grads[self.config.layer_id] = grad
 
-    def split_weights(self):
-        weights = [self._get_weights(i) for i in range(18)]
+    # def split_weights(self):
+    #     weights = [self._get_weights(i) for i in range(22)]
 
-        if self.config.layer_id == 0:
-            _shared_encdec_attn_kv_params["w"] = self._get_weights(18)
-            _shared_encdec_attn_kv_params["b"] = self._get_weights(19)
-        encdec_kvw = _shared_encdec_attn_kv_params["w"]
-        encdec_kvb = _shared_encdec_attn_kv_params["b"]
+    #     if self.config.layer_id == 0:
+    #         _shared_encdec_attn_kv_params["w"] = self._get_weights(19)
+    #         _shared_encdec_attn_kv_params["b"] = self._get_weights(20)
+    #     encdec_kvw = _shared_encdec_attn_kv_params["w"]
+    #     encdec_kvb = _shared_encdec_attn_kv_params["b"]
 
-        hs = self.config.hidden_size
-        offset = hs * hs * 2 * self.config.layer_id
-        encdec_kvw = encdec_kvw.data.narrow(0, offset, hs * hs * 2)
-        offset = hs * 2 * self.config.layer_id
-        encdec_kvb = encdec_kvb.data.narrow(0, offset, hs * 2)
-        weights += [encdec_kvw, encdec_kvb]
-        weights[0] = weights[0].view(-1, hs)
-        weights[2] = weights[2].view(-1, hs)
-        weights[6] = weights[6].view(-1, hs)
-        weights[8] = weights[8].view(-1, hs)
-        weights[12] = weights[12].view(-1, hs)
-        weights[14] = weights[14].view(hs, -1)
-        weights[18] = weights[18].view(-1, hs)
-        return weights
+    #     hs = self.config.hidden_size
+    #     offset = hs * hs * 2 * self.config.layer_id
+    #     encdec_kvw = encdec_kvw.data.narrow(0, offset, hs * hs * 2)
+    #     offset = hs * 2 * self.config.layer_id
+    #     encdec_kvb = encdec_kvb.data.narrow(0, offset, hs * 2)
+    #     weights += [encdec_kvw, encdec_kvb]
+    #     weights[0] = weights[0].view(-1, hs)
+    #     weights[2] = weights[2].view(-1, hs)
+    #     weights[6] = weights[6].view(-1, hs)
+    #     weights[8] = weights[8].view(-1, hs)
+    #     weights[12] = weights[12].view(-1, hs)
+    #     weights[14] = weights[14].view(hs, -1)
+    #     weights[19] = weights[19].view(-1, hs)
+    #     return weights
 
     def state_dict(self, destination=None, prefix="", keep_vars=False):
         destination = state_dict(
@@ -346,7 +423,7 @@ class LSTransformerDecoderLayer(nn.Module):
         return destination
 
     def forward(
-        self, decoder_states, encoder_out, encoder_padding_mask, cache, **kwargs
+        self, decoder_states, encoder_out, encoder_padding_mask, cache=None, **kwargs
     ):
         """
         decoder_states, [batch_size, trg_len, hidden_size] or [batch_size * beam_size, 1, hidden_size]
@@ -359,6 +436,8 @@ class LSTransformerDecoderLayer(nn.Module):
         """
         self.config.training = self.training
         self.config.is_grad_enabled = torch.is_grad_enabled()
+        self.config.quant_mode = self.quant_mode
+
         decoder_states = decoder_states.contiguous()
         # [s, b, h] -> [b, s, h]
         encoder_out = encoder_out.transpose(0, 1).contiguous()
@@ -421,15 +500,21 @@ class LSTransformerDecoderLayer(nn.Module):
         bs, sl, dim = decoder_states.size()
         if bs * sl > self.config.max_batch_tokens:
             raise ValueError(
-                f"Batch token numbers {bs * sl} exceeds the limit {self.config.max_batch_tokens}."
+                f"Batch token numbers {bs * sl} exceeds the limit"
+                f" {self.config.max_batch_tokens}."
             )
         if sl > self.config.max_seq_len:
             raise ValueError(
                 f"Sequence length {sl} exceeds the limit {self.config.max_seq_len}."
             )
-        assert encoder_out.size(0) == encoder_padding_mask.size(0) and encoder_out.size(
-            1
-        ) == encoder_padding_mask.size(1)
+        if len(encoder_padding_mask.size()) == 1:
+            assert encoder_out.size(0) == 1 and encoder_out.size(
+                1
+            ) == encoder_padding_mask.size(0)
+        else:
+            assert encoder_out.size(0) == encoder_padding_mask.size(
+                0
+            ) and encoder_out.size(1) == encoder_padding_mask.size(1)
         if cache is None:
             assert bs == encoder_out.size(0)
         else:
@@ -443,3 +528,9 @@ class LSTransformerDecoderLayer(nn.Module):
             cache_list,
         )
         return output.to(self.para)
+
+    def disable_quant(self):
+        self.quant_mode = False
+
+    def enable_quant(self):
+        self.quant_mode = True
